/*****************************************************  Common Functions  *****************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following functions are common to several Red Rapids applications.

	Rev History:	05-08-2015:		Initial release.

	Copyright 2015, Red Rapids, Inc.

*****************************************************  Common Functions  *****************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "adapter.h"
#include "flash.h"
#include "common_functions.h"

// Close the Red Rapids device on the PCI bus.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long BusClose(s_Adapter *p_HW)
{
	// Close the assigned device.
	return(Adapter_Close(p_HW));
}

// Open the Red Rapids device on the PCI bus.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long BusOpen(s_Adapter *p_HW)
{
	// Set all of the global interrupt mask register values in the hardware handle.
	p_HW->GlobalIntMask.BAR = BAR0;
	p_HW->GlobalIntMask.Offset = GLOBAL_INT_MASK;
	p_HW->GlobalIntMask.Disable = 0x0ull;
	// Open the assigned device.
	return(Adapter_Open(p_HW));
}

// Report the firmware revision date (MMDDYYYY).
// Returns the firmware revision date in hex format, but it is not a hex value (e.g. 04/27/2012 returns 0x04272012).
unsigned long FirmwareRevision(s_Adapter *p_HW)
{
	unsigned long long Value = 0;
	
	Read(p_HW, BAR2, IDENTIFICATION, &Value);
	
	return((unsigned long)(Value & 0x00000000FFFFFFFFull));
}

// Pause for the specified time in microseconds.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long MicrosecondTimer(unsigned long Time)
{
	// Pause for the specified time.
	return(Adapter_uSleep(Time));
}

// Report the model number of the device.
// Returns the model number in hex format, but it is not a hex value (e.g. Model 266 returns 0x266).
unsigned long ModelNumber(s_Adapter *p_HW)
{
	unsigned long long Value = 0;

	Read(p_HW, BAR2, IDENTIFICATION, &Value);

	return((unsigned long)((Value >> 32) & 0x0000000000000FFFull));
}

// Perform a 64-bit register read from the device at the specified BAR and address offset.
// Note: This function applies to both 32-bit and 64-bit operating systems.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long Read(s_Adapter *p_HW, unsigned long Bar, unsigned long Off, unsigned long long *p_Value)
{
	unsigned long Status = 0;

	// Read the requested offset and store the result in Value.
	Status = Adapter_Read64(p_HW, Bar, Off, p_Value);
	return(Status);
}

// Issue a software reset that is equivalent to a power-on reset.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long SoftwareReset(s_Adapter *p_HW)
{
	return(Write(p_HW, BAR0, SOFTWARE_RESET, 0x0ull));
}

// Perform a 64-bit target PCI write.
// Note: This function applies to both 32-bit and 64-bit operating systems.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long Write(s_Adapter *p_HW, unsigned long Bar, unsigned long Off, unsigned long long Value)
{
	// Write the contents of Value in the specified offset.
	return(Adapter_Write64(p_HW, Bar, Off, Value));
}
